/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 *
 * This file is part of Fanorona.
 *
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 *
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifndef _FANORONA_FLTK_H_
#define _FANORONA_FLTK_H_

#include <FL/Fl.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Group.H>

#include <fanorona/board.h>
#include <fanorona/player.h>

class FanoronaFlBoard: public Fl_Group {
public:
  typedef enum {
    Unmarked,
    Possible,
    Used
  } CellMark;

private:
  class Cell: public Fl_Widget {
    FanoronaFlBoard& parent;
    int col,row;
    CellMark curMark;
  protected:
    int handle(int event);
  public:
    void draw();
    Cell(int x, int y,
         int w, int h,
         FanoronaFlBoard& _parent,
         int _col, int _row);

    void mark(CellMark _mark);
  };

  Cell* cells[FANORONA_NUM_COLS][FANORONA_NUM_ROWS];

  FanoronaPlayer* player;

  struct {
    struct {
      int x,y;
    } topleft,topmid,topright,
      midleft,midmid,midright,
      botleft,botmid,botright;
  } cellCenters;

  struct {
    int x,y;
  } cellSteps;

  struct {
    struct {
      int col,row;
    } pendingFrom,pendingTo,pendingApproach,pendingWithdrawal;

    void reset()
    {
      pendingFrom.col=-1;
      pendingFrom.row=-1;
      pendingTo.col=-1;
      pendingTo.row=-1;
      pendingApproach.col=-1;
      pendingApproach.row=-1;
      pendingWithdrawal.col=-1;
      pendingWithdrawal.row=-1;
    }

  } moveState;

  typedef enum {
    Invalid,
    Approach,
    Withdrawal
  } Direction;

  FanoronaBoardWatcher boardWatcher;

  void markChoice(int col, int row,
                  Direction direction);
  void eraseChoices();

  int disabled;

  unsigned int delay;

protected:
  void draw();

public:
  FanoronaFlBoard(int x, int y,
                  int w, int h,
                  const char* label=0);

  FanoronaBoard& getBoard();

  void FanoronaFlBoard::cellClicked(Cell& cell,
                                    int col,
                                    int row);

  void cellChanged(int col,
                   int row, 
                   FanoronaBoardCellState state);
  void moveComputer();

  void newGame();

  virtual void gameOver(int humanWon);
  virtual void whoseTurn(int human);

  void start();
};

#endif /*_FANORONA_FLTK_H_*/
