/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 #
 * This file is part of Fanorona.
 #
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 #
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 #
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifndef _FANORONA_PLAYER_H_
#define _FANORONA_PLAYER_H_

#ifdef  __cplusplus
extern "C" {
#endif

#include <fanorona/board.h>

typedef struct _FanoronaPlayer FanoronaPlayer;

int fanoronaPlayerInit(FanoronaPlayer* player,
                       int isPlayer1,
                       int depth);
int fanoronaPlayerNewGame(FanoronaPlayer* player);
FanoronaPlayer* fanoronaPlayerNew(int isPlayer1,
                                  int depth);
void fanoronaPlayerFree(FanoronaPlayer* player);

int fanoronaPlayerSetDepth(FanoronaPlayer* player,
                           int depth);

FanoronaBoard* fanoronaPlayerGetBoard(FanoronaPlayer* player);

/* Copies board into player's internal board */
int fanoronaPlayerSetBoard(FanoronaPlayer* player,
                           const FanoronaBoard* board);

int fanoronaPlayerIsPlayer1(FanoronaPlayer* player);

int fanoronaPlayerMakeMove(FanoronaPlayer* player,
                           int* colFrom,
                           int* rowFrom,
                           int* colTo,
                           int* rowTo,
                           int* byApproach,
                           int* turnIsOver);
int fanoronaPlayerMoveMade(FanoronaPlayer* player,
                           int colFrom,
                           int rowFrom,
                           int colTo,
                           int rowTo,
                           int byApproach,
                           int* turnIsOver);

void* fanoronaPlayerGetState(FanoronaPlayer* player,
                             int* len);
int fanoronaPlayerRestoreState(FanoronaPlayer* player,
                               const void* data,
                               int len);

int fanoronaPlayerOtherPlayerResetPath(FanoronaPlayer* player);

int fanoronaPlayerOtherPlayerMoveValid(FanoronaPlayer* player,
                                       int colFrom,
                                       int rowFrom,
                                       int colTo,
                                       int rowTo,
                                       int* canApproach,
                                       int* canWithdraw);
int fanoronaPlayerOtherPlayerCanCapture(FanoronaPlayer* player,
                                        int colFrom,
                                        int rowFrom);

#ifdef  __cplusplus
}
#endif

#endif /*_FANORONA_PLAYER_H_*/
