/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 #
 * This file is part of Fanorona.
 #
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 #
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 #
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */
#ifndef __GTK_FANORONA_H__
#define __GTK_FANORONA_H__


#include <gdk/gdk.h>
#include <gtk/gtkadjustment.h>
#include <gtk/gtkwidget.h>

#include <fanorona/board.h>
#include <fanorona/player.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_FANORONA(obj)          GTK_CHECK_CAST (obj, gtk_fanorona_get_type (), GtkFanorona)
#define GTK_FANORONA_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_fanorona_get_type (), GtkFanoronaClass)
#define GTK_IS_FANORONA(obj)       GTK_CHECK_TYPE (obj, gtk_fanorona_get_type ())

guint gtk_fanorona_get_type(void);

typedef struct _GtkFanorona        GtkFanorona;
typedef struct _GtkFanoronaClass   GtkFanoronaClass;

typedef gboolean (*GtkFanoronaClickValidator)(GtkFanorona* fanorona,
                                              int col,
                                              int row,
                                              void* context);

typedef void* (*gtk_fanorona_get_state_method)(void* context,
                                               int* len);
typedef int (*gtk_fanorona_restore_state_method)(void* context,
                                                 const void* data,
                                                 int len);

typedef struct {
  int colFrom;
  int rowFrom;
  int colTo;
  int rowTo;
  int byApproach;
} GtkFanoronaMove;

typedef struct _GtkFanorona_SavedBoard {
  FanoronaBoard board;

  struct {
    int col,row;
  } pressedIn;

  FanoronaBoardCellState curPlayer;

  struct {
    int colFrom,rowFrom;
  } moveInProgress;

  gboolean possibles[FANORONA_NUM_COLS][FANORONA_NUM_ROWS];
  gboolean useds[FANORONA_NUM_COLS][FANORONA_NUM_ROWS];

  GtkFanoronaMove ambiguousMove;
  gboolean ambiguousMoveInProgress;

  struct _GtkFanorona_SavedBoard* next;
  struct _GtkFanorona_SavedBoard* prev;

  struct {
    void* data;
    int len;
  } externState;

  int turnNum;
} GtkFanorona_SavedBoard;

struct _GtkFanorona
{
  GtkWidget widget;

  FanoronaBoard* board;
  gboolean boardWasCreated;
  FanoronaBoardWatcher watcher;

  int cellsize,xmargin,ymargin;

  struct {
    int col,row;
  } pressedIn;

  struct {
    GdkColor empty,player1,player2,borders,pressed,possible;
  } colors;

  struct {
    GdkGC* empty;
    GdkGC* player1;
    GdkGC* player2;
    GdkGC* borders;
    GdkGC* pressed;
    GdkGC* possible;
  } gcs;

  FanoronaBoardCellState curPlayer;

  GtkFanorona_SavedBoard* savedBoards;
  GtkFanorona_SavedBoard* latestBoard;

  struct {
    int colFrom,rowFrom;
  } moveInProgress;

  struct {
    GtkFanoronaClickValidator function;
    void* context;
  } clickValidator;

  gboolean possibles[FANORONA_NUM_COLS][FANORONA_NUM_ROWS];
  gboolean useds[FANORONA_NUM_COLS][FANORONA_NUM_ROWS];

  GtkFanoronaMove ambiguousMove;
  gboolean ambiguousMoveInProgress;

  /*gboolean firstMoveOfTurn;*/

  int delay;

  struct {
    gtk_fanorona_get_state_method get;
    gtk_fanorona_restore_state_method restore;

    void* context;
  } stateMethods;
};

struct _GtkFanoronaClass {
  GtkWidgetClass parent_class;

  void (*clicked)(GtkWidget* widget,
                  gint col,
                  gint row,
                  gpointer data);
  void (*moved)(GtkFanorona* fanorona,
                const GtkFanoronaMove* move);
  void (*game_over)(GtkFanorona* fanorona,
                    gboolean player1Won);
  void (*prompt_for_direction)(GtkFanorona* fanorona);
  void (*report_whose_move)(GtkFanorona* fanorona,
                            gboolean player1sMove);
};


GtkWidget*     gtk_fanorona_new                    (FanoronaBoard* board);
FanoronaBoard* gtk_fanorona_get_board         (GtkFanorona      *fanorona);
void           gtk_fanorona_set_board         (GtkFanorona      *fanorona,
                                               FanoronaBoard* board);
void gtk_fanorona_invalidate_cell(GtkFanorona* fanorona,
                                  int col,
                                  int row);

void gtk_fanorona_set_color_empty(GtkFanorona* fanorona,
                                  const char* empty);
void gtk_fanorona_set_color_player1(GtkFanorona* fanorona,
                                    const char* player1);
void gtk_fanorona_set_color_player2(GtkFanorona* fanorona,
                                    const char* player2);
void gtk_fanorona_set_color_borders(GtkFanorona* fanorona,
                                    const char* borders);
void gtk_fanorona_set_color_pressed(GtkFanorona* fanorona,
                                    const char* pressed);
void gtk_fanorona_set_color_possible(GtkFanorona* fanorona,
                                    const char* possible);

void gtk_fanorona_set_colors(GtkFanorona* fanorona,
                             const char* empty,
                             const char* player1,
                             const char* player2,
                             const char* borders,
                             const char* pressed,
                             const char* possible,
                             gboolean invalidate);

int gtk_fanorona_get_delay(GtkFanorona* fanorona);
void gtk_fanorona_set_delay(GtkFanorona* fanorona,
                            int delay);

void gtk_fanorona_set_cur_player(GtkFanorona* fanorona,
                                 FanoronaBoardCellState state);
FanoronaBoardCellState
gtk_fanorona_get_cur_player(GtkFanorona* fanorona);

void gtk_fanorona_enable_moves(GtkFanorona* fanorona,
                               FanoronaPlayer* player);
void gtk_fanorona_new_game(GtkFanorona* fanorona);
void gtk_fanorona_undo(GtkFanorona* fanorona);
void gtk_fanorona_redo(GtkFanorona* fanorona);

void
gtk_fanorona_set_click_validator(GtkFanorona* fanorona,
                                 GtkFanoronaClickValidator clickValidator,
                                 void* context);
GtkFanoronaClickValidator
gtk_fanorona_get_click_validator(GtkFanorona* fanorona);
void*
gtk_fanorona_get_click_validator_context(GtkFanorona* fanorona);

void gtk_fanorona_clear_all_possibles(GtkFanorona* fanorona);
void gtk_fanorona_set_possible(GtkFanorona* fanorona,
                               int col,
                               int row,
                               gboolean possible);
gboolean gtk_fanorona_get_possible(GtkFanorona* fanorona,
                                   int col,
                                   int row);

void gtk_fanorona_clear_all_useds(GtkFanorona* fanorona);
void gtk_fanorona_set_used(GtkFanorona* fanorona,
                           int col,
                           int row,
                           gboolean used);
gboolean gtk_fanorona_get_used(GtkFanorona* fanorona,
                               int col,
                               int row);

void gtk_fanorona_set_state_methods(GtkFanorona* fanorona,
                                    void* context,
                                    gtk_fanorona_get_state_method get,
                                    gtk_fanorona_restore_state_method restore);

int gtk_fanorona_save(GtkFanorona* fanorona,
                      const char* configFile);
int gtk_fanorona_restore(GtkFanorona* fanorona,
                         const char* configFile);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_FANORONA_H__ */
/* example-end */
