/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 #
 * This file is part of Fanorona.
 #
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 #
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 #
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <errno.h>
#include <string.h>

#include "fakeBoard.h"

/* Function to initialize a board to a particular state that was
 *  giving me trouble (so I wouldn't have to play through the game
 *  each time).  Saved here for future just-in-case.
 */

static void fakeBoard1(FanoronaBoard* board)
{
  int col=0,row=0;

  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);

  row++;
  col=0;

  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);

  row++;
  col=0;

  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer1);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);

  row++;
  col=0;

  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);

  row++;
  col=0;

  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStateEmpty);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);
  fanoronaBoardSetCell(board,col++,row,fanoronaBoardCellStatePlayer2);

  row++;
  col=0;
}

int parsePrintedBoard(FanoronaBoard* board,
                      const char* printed)
{
  int col=0, row=0;

  while ((*printed) && (fanoronaBoardIsLegalPos(col,row)))
    {
      char cell=(*printed);
      FanoronaBoardCellState state;

      switch (cell)
        {
        case '.':
          state=fanoronaBoardCellStateEmpty;
          break;

        case '1':
          state=fanoronaBoardCellStatePlayer1;
          break;

        case '2':
          state=fanoronaBoardCellStatePlayer2;
          break;

        default:
          fprintf(stderr,
                  "parsePrintedBoard(): found '%c' where expecting cell\n",
                  cell);
          errno=EINVAL;
          return -1;
        }

      fanoronaBoardSetCell(board,col++,row,state);

      printed++;
      if ((*printed)=='\n')
        {
          col=0;
          row++;
          printed=strchr(printed+1,'\n');
          if (!printed)
            break;
        }
      printed++;
    }
}
