/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 #
 * This file is part of Fanorona.
 #
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 #
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 #
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <errno.h>
#include <fanorona/player.h>

#include "fakeBoard.h"

static int player1Move(FanoronaPlayer* player,
                       int* turnIsOver)
{
  int colFrom,rowFrom,colTo,rowTo,byApproach;

  if (fanoronaPlayerMakeMove(player,
                             &colFrom,&rowFrom,&colTo,&rowTo,&byApproach,
                             turnIsOver)
      <0)
    {
      int tmp=errno;
      fprintf(stderr,"fanoronaPlayerMakeMove() failed: %s\n",
              strerror(tmp));
      exit(2);
    }

  printf("Player 1 moved from (%d,%d) to (%d,%d), by %s\n",
         colFrom,rowFrom,colTo,rowTo,
         (byApproach ? "approach" : "withdrawal"));
}

static int player2Move(FanoronaPlayer* player,
                       int* turnIsOver)
{
  int colFrom,rowFrom,colTo,rowTo,byApproach;

  printf("Player 2's move:\n");

  do {
    printf("colFrom: ");
    fflush(stdout);
    scanf("%d",&colFrom);
    printf("rowFrom: ");
    fflush(stdout);
    scanf("%d",&rowFrom);
    printf("colTo: ");
    fflush(stdout);
    scanf("%d",&colTo);
    printf("rowTo: ");
    fflush(stdout);
    scanf("%d",&rowTo);
    printf("byApproach: ");
    fflush(stdout);
    scanf("%d",&byApproach);
  } while (byApproach<0);

  if (fanoronaPlayerMoveMade(player,
                             colFrom,
                             rowFrom,
                             colTo,
                             rowTo,
                             byApproach,
                             turnIsOver)
      <0)
    {
      int tmp=errno;
      fprintf(stderr,"fanoronaPlayerMoveMade() failed: %s\n",
              strerror(tmp));
      exit(3);
    }
}

int main(void)
{
  int isPlayer1sTurn=1;
  int numForPlayer1,numForPlayer2;

  FanoronaPlayer* player=fanoronaPlayerNew(1,3);
  if (!player)
    {
      int tmp=errno;
      fprintf(stderr,"fanoronaPlayerInit() failed: %s\n",
              strerror(tmp));
      exit(1);
    }

  /*parsePrintedBoard((FanoronaBoard*)fanoronaPlayerGetBoard(player),
                    ".-1-1-1-.-1-1-1-.\n"
                    "|\\|/|\\|/|\\|/|\\|/|\n"
                    ".-.-.-.-.-.-.-.-1\n"
                    "|/|\\|/|\\|/|\\|/|\\|\n"
                    ".-.-.-1-.-.-.-2-1\n"
                    "|\\|/|\\|/|\\|/|\\|/|\n"
                    ".-.-.-.-.-.-.-.-2\n"
                    "|/|\\|/|\\|/|\\|/|\\|\n"
                    "2-2-1-.-.-.-.-2-.\n");*/

  fanoronaBoardPrint(stdout,fanoronaPlayerGetBoard(player));
  do {
    int turnIsOver;
    if (isPlayer1sTurn)
      player1Move(player,&turnIsOver);
    else
      player2Move(player,&turnIsOver);

    if (turnIsOver)
      isPlayer1sTurn=!isPlayer1sTurn;

    fanoronaBoardPrint(stdout,fanoronaPlayerGetBoard(player));
    fanoronaBoardCountSides(fanoronaPlayerGetBoard(player),
                            &numForPlayer1,&numForPlayer2);
  } while ((numForPlayer1>0) && (numForPlayer2>0));
}
